package kpalin.alabra;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**  This is the visual representation of the chessboard.
 @author Kimmo Palin
*/

public class BoardGrid extends Panel implements Serializable{
  /** Board tiles where you can place the pieces.
   */
  private BoardTile [][] tiles;

  /** These are the tiles of the kings. Kind of cache to prevent
      searching through board after every move.
  */
  private BoardTile kingTiles[];

  /** These are names of the columns. No need to embed these in there.
   */
  final static char IDSTRS[]={'a','b','c','d','e','f','g','h'};

  /** Writing BoardGrid-object to ObjectOutput.
   */
  public void writeExternal(ObjectOutput out) throws IOException {
    for(int c=0;c<ChessSettings.COLUMNS;c++) {
      for(int r=0;r<ChessSettings.ROWS;r++) {
	out.writeObject(getTile(c,r));
      }
    }
  }

  /** Reading BoardGrid from ObjectInput.
   */
  public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException{
    for(int c=0;c<ChessSettings.COLUMNS;c++) {
      for(int r=0;r<ChessSettings.ROWS;r++) {
	tiles[c][r]=(BoardTile)in.readObject();
      }
    }
  }

  /** Creates new chessboard.
      @param listen Listener for the tiles.
  */
  BoardGrid(MouseListener listen) {
    tiles=new BoardTile[ChessSettings.ROWS][ChessSettings.COLUMNS];

    setLayout(new GridLayout(9,9));
    kingTiles=new BoardTile [2];

    /* Here we flip the board so white is down and black is up but still
       we can manage as (0,0) is a1 and (1,0) is b1*/

    for(int r=ChessSettings.ROWS-1;r>=0;r--) {
      add(new Label(""+(r+1)));
      for(int c=0;c<ChessSettings.COLUMNS;c++) {
	tiles[c][r]=new BoardTile(c,r);
	tiles[c][r].addMouseListener(listen);
	add(tiles[c][r]);
      }
    }

    add(new Panel()); /* Just an empty panel to pad column ids. */
    for(int c=0;c<8;c++)
      add(new Label(""+IDSTRS[c],Label.CENTER));
  }

  /** Fast way of find out kings position.
      @param side Side of the wanted king.
      @return Tile where the king is.
  */
  public BoardTile kingTiles(int side) {
    return kingTiles[side];
  }

  /** An accessor to control pieces on the board.
      @param row Row of the piece.
      @param col Column of the piece.
      @param p The piece you want to put there. null for clearing position.
   */
  public  void setPiece(int row, int col, Piece p) {
    if(p!=null && p.getType()==Piece.KING)
      kingTiles[p.getSide()]=tiles[row][col];

    tiles[row][col].setPiece(p);
  }

  /** Sets the grid to the standing defined in ChessSettings.
      @param set The settings to be set on the board.
  */
  void setBoard(ChessSettings set) {
    for(int r=0;r<ChessSettings.ROWS;r++) {
      for(int c=0;c<ChessSettings.COLUMNS;c++) {
	setPiece(c,r,set.pieceAt(r,c));
      }
    }
  }

  /** An accessor to fetch piece in specific location. Throws
      exception if coordinates are invalid.
      @param col Column of the tile.
      @param row Row of the tile.
      @return Returns piece on the tile, null if theres not a piece.
  */
  public  Piece getPiece(int col,int row) {
    return tiles[col][row].getPiece();
  }
  /** An accessor to fetch specific tile. Throws
      exception if coordinates are invalid. Please try
      to avoid using this but this is needed for king cache
      in ChessPlay
      @param col Columun of the tile.
      @param row Row of the tile.
      @return Returns tile.
  */
   BoardTile getTile(int col,int row) {
    return tiles[col][row];
  }
}

