package kpalin.alabra;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** This is the key class of the whole program. This is attached on
    application window or an applet panel and the whole chess stuff is
    under this class.

    @author Kimmo Palin
*/

public class ChessPlay extends Panel implements MouseListener,ChessCtrlCallback{
  /** The panel that holds the chess board.
   */
  private BoardGrid grid;
  /** Initial settings got with an implementation specific way.
   */
  private final ChessSettings initSet;

  /** The panel that holds all control etc. stuff on the screen.
   */
  private CtrlPanel ctrl;

  /** Process click-events.
   */
  private boolean processClicks=true;

  /** Whos turn is it?.
      @see Piece.WHITE
      @see Piece.BLACK
      @see Piece.GAMEOVER
   */
  private int whosTurn;


  /** This is an instance of pawn promotion dialog. Set in pawnPromote
      and null:ed in update or resetGame.
    */
  private PawnPromote pPromotion=null;

  /** The tile where is the piece to be moved. Not really good place
      for this stuff but can't help it. Only mouseClicked can use this!
      @see mouseClicked
   */
  private BoardTile startTile;

  /** The previous move. This is needed for the en'passant.
   */
  private BoardTile prevTarget;

  /** Possibility of fortification. Left dimension is Piece.WHITE or 
      Piece.Black, right Piece.LONGFORT or Piece.SHORTFORT
  */

  private boolean canFortify[][]={{true,true},{true,true}};

  /** This is the data about tiles that can be moved to block a check.
      If there is no current threat, this will be null.
   */
  private Vector checkBlockers;

  /** Creates new chess play object.
   */
  ChessPlay(ChessSettings set) {
    initSet=set;

    setLayout(new GridLayout(1,2));

    ctrl=new CtrlPanel(this,set.getMoveLimit(),set.getTimeLimit());
    ctrl.setFont(initSet.getFont());

    grid=new BoardGrid(this);
    grid.setBoard(initSet);
    whosTurn=initSet.getStartSide();

    add(grid);
    add(ctrl);
    
  }

  /** Reset the game*/
  public void resetGame() {
    grid.setVisible(false);
    grid.setBoard(initSet);
    ctrl.resetAll();
    try {
      startTile.setSelected(false);
      startTile=null;
    } catch(NullPointerException e) {}
    

    canFortify=initSet.fortClone();
    whosTurn=initSet.getStartSide();

    if(pPromotion!=null) {
      pPromotion.setVisible(false);
      pPromotion.getObservable().deleteObserver(this);
      pPromotion=null;
    }

    checkBlockers=null;
    processClicks=true;
    grid.setVisible(true);
  }

  /** Starts pawnPromotion procedure. This is a bit dirty because
      we have to artificially block input to ChessPlay as long 
      as this goes on. The control is returned to ChessPlay
      after update.
      @see update
  */
  void promotePawn(int side, BoardTile from, BoardTile to) {
    pPromotion=new PawnPromote(side,from,to);

    pPromotion.getObservable().addObserver(this);
    pPromotion.setVisible(true);
      
    processClicks=false;
  }

  /** Listener for pawn promotion.
      @param o the observable object
      @param arg an argument (not used)
    */
  public void update(Observable o, Object arg) {

    if(o instanceof PromotionListen)
      finishPromotion((PromotionListen)o);
    else if(o instanceof ClockListen) {
      int winner=((ClockListen)o).getPlayer()%Piece.GAMEOVER==Piece.WHITE
	?Piece.BLACK:Piece.WHITE;

      recordGameOver(winner);

      gameOver(winner);
    }
  }

  /** Finalizes the pawn promotion.
      @param plisten The communication object towards PawnPromotion.
  */
  private void finishPromotion(PromotionListen plisten) {
    plisten.deleteObserver(this);

    processClicks=true;
    movePiece(plisten.FROM,plisten.TO,plisten.getPiece());
    changeTurn();
    pPromotion=null;
  }

  /** Listens for the reset button on ctrlPanel.
   */
  public void actionPerformed(ActionEvent e) {
    resetGame();
  }

  public void mouseEntered(MouseEvent e) { return; }
  public void mouseExited(MouseEvent e) { return; }
  public void mousePressed(MouseEvent e) { return; }
  public void mouseReleased(MouseEvent e) { return; }

  /** This is the callback method for the BoardTiles. This is the best
      place for this because it needs connections to ctrl (change turn)
      and grid (information about the board status)
   */
  public void mouseClicked(MouseEvent e) {
    if(processClicks==false)
      return;

    if(whosTurn>=Piece.GAMEOVER) {
      System.err.println("Inconsistency of program settings!\nPlease, reset.");
      return;
    }

    BoardTile source=(BoardTile)e.getComponent();


    if(startTile!=null) {
      /* this is the second click of the move. (the target tile)*/
      makeMove(startTile,source);
    } else if( source.getPiece()==null || 
	       whosTurn!=source.getPiece().getSide() ||
	       !canMove(source) ||
	       (checkBlockers!=null && !checkBlockers.contains(source))) {
      return; /* somehow invalid tile*/
    } else {
      startTile=source;
      source.setSelected();
    }
  }

  /** Checks if side is on mate.
      @param side Side which can be on mate.
      @return true if side is on mate, false if not.
  */
  private boolean checkForMate(int side) {
    BoardTile threat;
    Vector baddies=getNChecks(side,2,grid.kingTiles(side));

    //The king is not in check.
    if(baddies.size()==0 )
      return false;

    threat=(BoardTile)baddies.firstElement();  // onlyElement()
    this.checkBlockers=whoCanBlock(grid.kingTiles(side),threat,-1);


    //King can run and hide.
    if(canKingMove(grid.kingTiles(side))) {
      checkBlockers.addElement(grid.kingTiles(side));
      return false;
    }

    /** Come to think of it, this is impossible situation. */
    if(baddies.size()>1)
      return true; //mate! We can't block two threats on one move.


    return !(checkBlockers.size()>0);
  }

  /** Finds out the pieces that can come to block a check.
      @param kingTile The tile where's the threatend king
      @param threat The tile where is the agressor. (Piece must be there)
      @param limit if >0 return max limit threats <=0 return all threats.

      @return Vector of the tiles from where pieces can come blocking. (not null)
  */      
  private Vector whoCanBlock(BoardTile kingTile,BoardTile threat, int limit) {
    int deltax;
    int deltay;
    int dist=Math.max(Math.abs(threat.ypos-kingTile.ypos),
		      Math.abs(threat.xpos-kingTile.xpos))+1;

    //Side things here are from threat so we don't need a piece in kingTile
    int ourSide=(threat.getPiece().getSide()==Piece.WHITE?
		 Piece.BLACK:Piece.WHITE);
    Vector blockers=new Vector();

    int pawnRowDelta=(threat.getPiece().getSide()==Piece.WHITE?1:-1);

    try { //The threat comes from direction (deltax,deltay)
      deltax=(threat.xpos-kingTile.xpos)/Math.abs(threat.xpos-kingTile.xpos);
    } catch(ArithmeticException e) { deltax=0; }
    try {
      deltay=(threat.ypos-kingTile.ypos)/Math.abs(threat.ypos-kingTile.ypos);
    } catch(ArithmeticException e) { deltay=0; }


    if(limit<=0)
      limit=Integer.MAX_VALUE-1;

    //Only way of blocking knight is to eat it.
    if(threat.getPiece().getType()==Piece.KNIGHT) {
      otherThreat(blockers,threat.getPiece().getSide(),limit,threat);
      knightThreat(blockers,threat.getPiece().getSide(),limit,threat);
      pawnThreat(blockers,threat.getPiece().getSide(),limit,threat);

      return blockers;
    }


    try {
      //These aren't threats but movement possibilities.
      // i.e. Would there be a threat for the other side.
      for(int pos=1;blockers.size()<limit && pos<dist ;pos++) {
	otherThreat(blockers,threat.getPiece().getSide(),limit,
		    grid.getTile(kingTile.xpos+pos*deltax,kingTile.ypos+pos*deltay));

	//This isn't too clean but I still want to use the threat-methods.
	blockers.removeElement(grid.kingTiles(ourSide));

	knightThreat(blockers,threat.getPiece().getSide(),limit,
		     grid.getTile(kingTile.xpos+pos*deltax,kingTile.ypos+pos*deltay));

	try {
	  //Can't use pawnThreat here because of the pawn movement rules.
	  if(grid.getPiece(kingTile.xpos+pos*deltax,kingTile.ypos+pos*deltay
			+pawnRowDelta).getType()==Piece.PAWN && pos<(dist-1)) {
	    blockers.addElement(grid.getTile(kingTile.xpos+pos*deltax,
					     kingTile.ypos+pos*deltay+pawnRowDelta));
	  }
	} catch(Exception e) {} //ArrayIndex or NullPointer

	//Can pawn eat the aggressor?
	if(pos==(dist-1))
	    pawnThreat(blockers,threat.getPiece().getSide(),limit, threat);

      } // End of for() loop
    } catch(ArrayIndexOutOfBoundsException e) { 
      System.err.println("ArrayError in ChessPlay! This is serious!");
    }
    return blockers;
  }


  /** Takes care of the boring routines when we change turn.
   */
  private void changeTurn() {
    startTile.setSelected(false); 
    startTile=null;

    if(whosTurn>=Piece.GAMEOVER)
      return;

    if(whosTurn==Piece.WHITE)
      whosTurn=Piece.BLACK;
    else
      whosTurn=Piece.WHITE;

    ctrl.resumeTimer(whosTurn);

  }


  /** Validate and make the move.
      @param from The tile where the move starts.
      @param target The target tile where to move the piece.
  */
  private void makeMove(BoardTile from,BoardTile target) {
    int fromside=from.getPiece().getSide();

    if(target==from)
      return;


    BoardTile threat=blocksThreatAt(fromside,from);

    //eliminates threat-->must eliminate it after the move.
    // blocks it or eats it.
    if(threat!=null) {
      BoardTile newThreat=blocksThreatAt(fromside,3,target);

      // This is a quick and dirty hack.
      if(newThreat==null && target.getPiece()!=null && 
	 target.getPiece().getSide()!=fromside) {
	BoardTile king=grid.kingTiles(fromside);
	int deltax,deltay;

	//we are trying to eat it.
	if(king.xpos==target.xpos || king.ypos==target.ypos ||
	   Math.abs(king.xpos-target.xpos)==Math.abs(king.ypos-target.ypos)) {
	  
	  try { //The threat comes from direction (deltax,deltay)
	    deltax=(target.xpos-king.xpos)/Math.abs(target.xpos-king.xpos);
	  } catch(ArithmeticException e) { deltax=0; }
	  try {
	    deltay=(target.ypos-king.ypos)/Math.abs(target.ypos-king.ypos);
	  } catch(ArithmeticException e) { deltay=0; }
	  newThreat=grid.getTile(from.xpos+deltax,from.ypos+deltay);

	  /*if((from.xpos+deltax)!=threat.xpos || (from.ypos+deltay)!=threat.ypos)
	    return;
	  else newThreat=target;*/ 
	}
      //doesn't block a threat in same direction anymore
      }
      if(newThreat!=threat && (newThreat==null || 
	  //Threat direction check
	  (threat.xpos-from.xpos) != (newThreat.xpos-target.xpos) || 
	  (threat.ypos-from.ypos) != (newThreat.ypos-target.ypos))) {
	//Trying to give in check. (towards threat)
	return;
      }
    }
    

  
    /* check breaking 
     if 
     We are not moving king,
     There is check situation,
     A position where we are moving, does not eliminate a threat,
     nor eat the threatening piece.
     then return
     else null the check situation.
    */
    if(from.getPiece().getType()!=Piece.KING && checkBlockers!=null && 
       blocksThreatAt(fromside,target)==null &&
       isCheckPos(fromside,grid.kingTiles(fromside))!=target) 
      return;
    else
      checkBlockers=null;


    switch(from.getPiece().getType()) {
    case Piece.PAWN:
      if(!validPawnMove(from,target)) 
	return;
      if((from.getPiece().getSide()==Piece.WHITE && target.ypos==7) ||
	 (from.getPiece().getSide()==Piece.BLACK && target.ypos==0)){
	promotePawn(from.getPiece().getSide(),from,target);
	return;
      }
      break;
    case Piece.ROOK:
      if(from.xpos==0 && // Check that rook hasn't moved
	 from.ypos==grid.kingTiles(from.getPiece().getSide()).ypos)
	canFortify[from.getPiece().getSide()][Piece.LONGFORT]=false;
      else
	canFortify[from.getPiece().getSide()][Piece.SHORTFORT]=false;

      if(!validRookMove(from,target))
	return;
      break;
    case Piece.BISHOP:
      if(!validBishopMove(from,target))
	return;
      break;
    case Piece.QUEEN:
      if(!validQueenMove(from,target))
	return;
      break;
    case Piece.KNIGHT:
      if(!validKnightMove(from,target)) 
	return;
      break;
    case Piece.KING:
      if(validFort(from,target)) {
	int rookFrom=0,rookTo=3;
	if(from.xpos<target.xpos) { //SHORTFORT
	  rookFrom=7;
	  rookTo=5;
	}
	grid.setPiece(rookTo,target.ypos,grid.getPiece(rookFrom,target.ypos));
	grid.setPiece(rookFrom,target.ypos,null);

      } else if(!validKingMove(from,target)) {
	return;
      }
    
      canFortify[from.getPiece().getSide()][Piece.LONGFORT]=false;
      canFortify[from.getPiece().getSide()][Piece.SHORTFORT]=false;

      break;
    default:
      System.err.println("No such Piece! Something is terribly bad!");
      return;
    }

    movePiece(from,target);
      
    changeTurn();
  }

  /** Tries to clean up after stoping applet.
   */
  public void killAll() {
    ctrl.killAll();
  }

  /** An assistant method to validate moves. Analyses how a piece in a
      specific tile blocks threats to king. The return tile is always
      in (1,1)-(-1,-1) box near the given tile.

      @param side The moving side.
      @param tile The tile whose blocking we're analyzing. 
      No need to have piece in here.

      @return null if piece in tile doesn't block any threats, otherwise
      the tile in the direction of the threat.
  */
  private BoardTile blocksThreatAt(int side,BoardTile tile) {
    return blocksThreatAt(side,2,tile);
  }
  /** An assistant method to validate moves. Analyses how a piece in a
      specific tile is possibly blocking threats to king. The return tile
      is always in (1,1)-(-1,-1) box near the given tile.

      @param side The moving side.
      @param jumpcount Number of pieces +1 that can be between king and threat.
      @param tile The tile whose blocking we're analyzing. 
      No need to have piece in here.

      @return null if piece in tile doesn't block any threats, otherwise
      the tile in the direction of the threat.
  */
  private BoardTile blocksThreatAt(int side, int jumpcount,BoardTile tile) {
    BoardTile king=grid.kingTiles(side);
    BoardTile threatTile=null;
    int deltax;
    int deltay;

    if(king.xpos==tile.xpos || king.ypos==tile.ypos || // maybe blocking.
       Math.abs(king.xpos-tile.xpos)==Math.abs(king.ypos-tile.ypos)) {

      try { //The threat comes from direction (deltax,deltay)
	deltax=(tile.xpos-king.xpos)/Math.abs(tile.xpos-king.xpos);
      } catch(ArithmeticException e) { deltax=0; }
      try {
	deltay=(tile.ypos-king.ypos)/Math.abs(tile.ypos-king.ypos);
      } catch(ArithmeticException e) { deltay=0; }
      
      // Trying to block with king?!?
      if(deltax==0 && deltay==0)
	return null;

      try {  // huge try-clause for ArrayIndexOutOfBoundsException
	// We start from king towards the requested tile so we can be sure
	// that tile is the only-one blocking the threat. i.e. when we jump
	// over it, the next piece will be enemy
	int len=1;
	for(int jumpPiece=0;jumpPiece<jumpcount;jumpPiece++) {
	  while(grid.getPiece(king.xpos+deltax*len,king.ypos+deltay*len)==null 
		&& grid.getTile(king.xpos+deltax*len,king.ypos+deltay*len)!=tile )
	      len++;
	  
	  threatTile=grid.getTile(king.xpos+deltax*len,
				  king.ypos+deltay*len);

	  // if we don't reach tile first, it is not blocking.
	  // We jump over own pieces and the tile we are looking for.
	  if(jumpPiece<(jumpcount-1) && threatTile!=tile && 
	     threatTile.getPiece()!=null && 
	     threatTile.getPiece().getSide()!=side) 
	    return null;
	  if(jumpPiece<(jumpcount-1))
	    len++; //jump over the blocker.
	}


	// Is there anything to be afraid of?
	if(threatTile.getPiece().getSide() != side &&
	   (threatTile.getPiece().getType() == Piece.QUEEN ||
	    (threatTile.getPiece().getType() == Piece.ROOK && 
	     (deltax==0|| deltay==0) ) ||
	    (threatTile.getPiece().getType() == Piece.BISHOP && 
	     (deltax!=0 && deltay!=0) ) ) ) {
	  return grid.getTile(tile.xpos+deltax,tile.ypos+deltay);
	}
      } catch(ArrayIndexOutOfBoundsException e) { }
    }

    return null;
  }
  

  /** Checks validity of en'passant-move
      @param from Tile where the move starts. Here must be a piece.
      @param target Tile where player is moving a piece.
  */
  private boolean validEnPassant(BoardTile from,BoardTile target) {

    try {
      if(from.getPiece().getType()!=Piece.PAWN ||
	 prevTarget.getPiece().getType()!=Piece.PAWN)
	return false; //Both must be pawns.
    } catch(NullPointerException e) { 
      return false;
    } // The first move throws exception

    if(Math.abs(target.xpos-from.xpos)!=1 ||
       target.xpos!=prevTarget.xpos)
      return false; //Columns must match
    
    if(from.getPiece().getSide()==Piece.WHITE &&
	(from.ypos!=4 || prevTarget.ypos!=4 ||
	 target.ypos!=5))
      return false;  // Row stuff for white

    if(from.getPiece().getSide()==Piece.BLACK &&
	(from.ypos!=3 || prevTarget.ypos!=3 ||
	 target.ypos!=2))
      return false;  // Row stuff for black


    if(from.getPiece().getSide()==prevTarget.getPiece().getSide())
      System.err.println("We're in serious trouble, double move for one side!");

    return true;
  }

  /** Checks the validity of a move for a Pawn.
      @param target The target tile.
  */
  private boolean validPawnMove(BoardTile from,BoardTile target) {

    if(validEnPassant(from, target))
       return true;
    

    if(((from.ypos+1)==target.ypos && //WHITE going forward
	from.getPiece().getSide()==Piece.WHITE) || 
       ((from.ypos-1)==target.ypos && //BLACK going forward
	from.getPiece().getSide()==Piece.BLACK))  {

      //Straight
      if(from.xpos==target.xpos && target.getPiece()==null)
	return true;
      //Eating
      else if(Math.abs(from.xpos-target.xpos)==1 && 
	      target.getPiece()!=null && target.getPiece().getSide()!=
	      from.getPiece().getSide()) 
	return true;
    }

    else if(target.xpos==from.xpos && 
	    Math.abs(target.ypos-from.ypos)==2 &&
	    ((target.ypos == 3 && //WHITE going forward 2 steps
	      from.getPiece().getSide()==Piece.WHITE) || 
	     (target.ypos == 4 && //BLACK going forward 2 steps
	      from.getPiece().getSide()==Piece.BLACK))) 
      return true;

    return false;
  }

  /** Checks the validity of a move for a Bishop.
      @param target The target tile.
  */
  private boolean validBishopMove(BoardTile from,BoardTile target) {
    if(Math.abs(target.xpos-from.xpos)!=
       Math.abs(target.ypos-from.ypos))
      return false;

    int delta=Math.abs(target.xpos-from.xpos);

    if(delta==0)  // We'll never reach here but in any case..
      return false;

    int signx=(target.xpos-from.xpos)/delta;
    int signy=(target.ypos-from.ypos)/delta;


    try {  // Check that we don't eat own piece
      if(target.getPiece().getSide()==from.getPiece().getSide())
	return false;
    } catch(NullPointerException e) {}

    return validStraightMove(from,signx,signy,delta);
  }

  /** Checks for threatening knights.
      @param baddies A vector where to add the threats.
      @param side The kings side. (Piece.WHITE or Piece.BLACK)
      @param kingPos The tile whose threat to analyse. (note: no need for a
      King to be in this tile.)
      @param count Maximum number of threat-tiles to return
  */
  private void knightThreat(Vector baddies,int side, int count,BoardTile kingPos) {
    Piece tmp;

    /* Check for threatening knights*/
    for(int i=0;i<Piece.knightMoves.length && baddies.size()<count;i++) {
      try {
	tmp=grid.getPiece(kingPos.xpos+Piece.knightMoves[i][0],
			  kingPos.ypos+Piece.knightMoves[i][1]);
	if(tmp.getType()==Piece.KNIGHT && tmp.getSide()!=side)
	  baddies.addElement(grid.getTile(kingPos.xpos+Piece.knightMoves[i][0],
					  kingPos.ypos+Piece.knightMoves[i][1]));

      } catch(Exception e) {} //NullPointer or ArrayIndexOutOfBounds
    }

  }

  /** Checks for threatening pawns.
      @param baddies A vector where to add the threats.
      @param side The kings side. (Piece.WHITE or Piece.BLACK)
      @param kingPos The tile whose threat to analyse. (note: no need for a
      King to be in this tile.)
      @param count Maximum number of threat-tiles to return
  */
  private void pawnThreat(Vector baddies,int side,int count,BoardTile kingPos) {
    int pawnRow=(side==Piece.WHITE?1:-1) + kingPos.ypos;
    Piece tmp;
    int pawnEat[]={-1,1};

    for(int eatDir=0;eatDir<2 && baddies.size()<count;eatDir++) {
      try {
	tmp=grid.getPiece(kingPos.xpos+pawnEat[eatDir],pawnRow);
	if(tmp.getSide()!=side && tmp.getType()==Piece.PAWN)
	  baddies.addElement(grid.getTile(kingPos.xpos+pawnEat[eatDir],pawnRow));
      } catch(Exception e) {} //NullPointer or ArrayIndexOutOfBounds
    }
  }

  /** Checks for threatening rooks, bishops, and queen.
      @param baddies A vector where to add the threats.
      @param side The kings side. (Piece.WHITE or Piece.BLACK)
      @param kingPos The tile whose threat to analyse. (note: no need for a
      King to be in this tile.)
      @param count Maximum number of threat-tiles to baddies.
  */
  private void otherThreat(Vector baddies,int side,int count, BoardTile kingPos) {

    int cords[][]=Piece.rookMoves;
    int dist,i;
    Piece tmp;


    for(i=0;i<cords.length && baddies.size()<count;i++) {
      try {
	tmp=null;             //King doesn't count as blocking piece.
	for(dist=0;tmp==null||(tmp.getType()==Piece.KING && tmp.getSide()==side);){
	  dist++;
	  tmp=grid.getPiece((kingPos.xpos+cords[i][0]*dist),
			    (kingPos.ypos+cords[i][1]*dist));
	}
	//If its threat, add to baddies.
	if(tmp.getSide()!=side && (tmp.getType()==Piece.ROOK || 
				   tmp.getType()==Piece.QUEEN ||
				   (tmp.getType()==Piece.KING &&dist==1)))
	  baddies.addElement(grid.getTile((kingPos.xpos+cords[i][0]*dist),
					  (kingPos.ypos+cords[i][1]*dist)));

      } catch(ArrayIndexOutOfBoundsException e) {} 
    }


    cords=Piece.bishopMoves;

    for(i=0;i<cords.length && baddies.size()<count;i++) {
      try {
	tmp=null;             //King doesn't count as blocking piece.
	for(dist=0;tmp==null||(tmp.getType()==Piece.KING && tmp.getSide()==side);){
	  dist++;
	  tmp=grid.getPiece((kingPos.xpos+cords[i][0]*dist),
			    (kingPos.ypos+cords[i][1]*dist));
	}
	//If its threat, add to baddies.
	if(tmp.getSide()!=side && (tmp.getType()==Piece.BISHOP || 
				   tmp.getType()==Piece.QUEEN ||
				   (tmp.getType()==Piece.KING &&dist==1)))
	  baddies.addElement(grid.getTile((kingPos.xpos+cords[i][0]*dist),
					  (kingPos.ypos+cords[i][1]*dist)));

      } catch(ArrayIndexOutOfBoundsException e) {} 
    }


  }

  /** Finds all checks.
      @param side The kings side. (Piece.WHITE or Piece.BLACK)
      @param kingPos The tile whose threat to analyse. (note: no need for a
      King to be in this tile.)
      @param count How many threats to return.
      @return A Vector of threatening BoardTiles. Maximum size count.
  */
  private Vector getNChecks(int side, int count, BoardTile kingPos) {
    Vector baddies=new Vector(count);

    if(count<1)  //Exception ???? Don't think so. 
      return baddies;

    otherThreat(baddies,side,count,kingPos);

    knightThreat(baddies,side,count,kingPos);

    pawnThreat(baddies,side,count,kingPos);

    return baddies;
  }

  /** Checks for checks.
      @param side The kings side. (Piece.WHITE or Piece.BLACK)
      @param kingPos The tile whose threat to analyse. (note: no need for a
      King to be in this tile.)
      @return The tile of the threat or null.
  */
  private BoardTile isCheckPos(int side, BoardTile kingPos) {
    try {
      return (BoardTile)getNChecks(side,1,kingPos).firstElement();
    } catch(NoSuchElementException e) {
      return null;
    }
  }

  
  /** Checks the validity of a fortification.
      @param from Starting position of the king.
      @param target The target tile.
      @return true if can move, false if not
  */
  private boolean validFort(BoardTile from,BoardTile target) {
    int ourSide=from.getPiece().getSide();
    int kingsDelta=Math.abs(from.xpos-target.xpos);
    int route;

    if(from.getPiece().getType()!=Piece.KING)
      return false;
    if(from.ypos!=target.ypos)
      return false;
    if(kingsDelta!=2)
      return false;

    // direction of kings movement.
    kingsDelta=(target.xpos-from.xpos)/kingsDelta;

    if((kingsDelta>0 && !canFortify[ourSide][Piece.SHORTFORT]) ||
       (kingsDelta<=0 && !canFortify[ourSide][Piece.LONGFORT]))
      return false;

    if(isCheckPos(ourSide,from)!=null)
      return false;

    try {
    // No Pieces in between and no threats on first two tiles.
      for(route=1;
	  grid.getPiece(from.xpos+kingsDelta*route,target.ypos)==null &&
	    (route>2 || 
	     isCheckPos(ourSide,grid.getTile(from.xpos+kingsDelta*route,
					     target.ypos))== null);route++ )
	{ }
    } catch(ArrayIndexOutOfBoundsException e) { return false; }		    

    try {
      if(grid.getPiece(from.xpos+kingsDelta*route,target.ypos).getType()
	 != Piece.ROOK)
	return false;
    }catch(NullPointerException e) {return false;}

    return true;
  }
  /** Checks the validity of a move for a King.
      @param from Starting position.
      @param target The target tile.
      @return true if can move, false if not
  */

  private boolean validKingMove(BoardTile from,BoardTile target) {

    if(Math.abs(target.xpos-from.xpos)>1 ||
       Math.abs(target.ypos-from.ypos)>1) 
      return false;
    try {
      if(target.getPiece().getSide()==from.getPiece().getSide())
	return false;
    }catch(NullPointerException e) {  }


    if(isCheckPos(from.getPiece().getSide(),target)!=null)
      return false;

    return true;
  }

  /** Checks the validity of a move for a Knight.
      @param target The target tile.
  */
  private boolean validKnightMove(BoardTile from,BoardTile target) {
    int dx=Math.abs(target.xpos-from.xpos);
    int dy=Math.abs(target.ypos-from.ypos);

    try {
    return (((dy==1 && dx==2)||(dy==2&&dx==1)) && // correct location
	    (target.getPiece()==null || // empty tile     //or enemy piece
	    target.getPiece().getSide()!=from.getPiece().getSide()));
    } catch(NullPointerException e) {
      System.err.println("Trouble with validKnightMove: "+e);
      return false; 
    }

  }

  /** Checks the validity of a move for a Queen.
      @param from Start tile of the move.
      @param target The target tile.
  */
  private boolean validQueenMove(BoardTile from,BoardTile target) {
    return validRookMove(from,target)||validBishopMove(from,target);
  }

  /** Checks the validity of a move for a Rook.
      @param target The target tile.
  */
  private boolean validRookMove(BoardTile from,BoardTile target) {
    int signx,signy,delta;

    try {  // Check that we don't eat our own piece
      if(target.getPiece().getSide()==
	 from.getPiece().getSide())
	return false;
    } catch(NullPointerException e) { /* Empty tile */ }
    

    if(target.xpos==from.xpos )
      return validStraightMove(from,0,(target.ypos-from.ypos)/
			       Math.abs(target.ypos-from.ypos),
			       Math.abs(target.ypos-from.ypos));
    else if(target.ypos==from.ypos)
      return validStraightMove(from,(target.xpos-from.xpos)/
			       Math.abs(target.xpos-from.xpos),0,
			       Math.abs(target.xpos-from.xpos));
			       
    return false;

  }


  /** This is a helpper method for pieces that move straight lines and
      diagonals.
      @param signx Direction to move on x acsis.
      @param singy Direction to move on y acsis.
      @param delta Length of the move. (Measured in (signx,signy))
      @return true if there's no pieces on the way.
  */
  private boolean validStraightMove(BoardTile from,int signx,int signy,int delta) {

    try {
      for(int i=1;i<Math.abs(delta);i++) {  //No pieces in between
	if(grid.getPiece(from.xpos+signx*i,from.ypos+signy*i)!=null)
	  return false;
      }
    } catch(ArrayIndexOutOfBoundsException e) { 
      System.err.println("Something seriously wrong in validStraightMove:"+e);
    }

    return true;
  
  }

  /** Moves the piece to target without any
      checking of validity of the move. This also removes eaten piece
      in en passant move. The move record is updated here because
      this is the only place we have enough information at hand.

      @param from Starting tile of the move.
      @param target The target tile where the piece is to be moved.
  */
  private void movePiece(BoardTile from,BoardTile target) {
    movePiece(from,target,null);
  }
  /** Moves the piece to target without any
      checking of validity of the move. This also removes eaten piece
      in en passant move. The move record is updated here because
      this is the only place we have enough information at hand.

      @param from Starting tile of the move.
      @param target The target tile where the piece is to be moved.
      @param promoteTo A piece, to which the moving piece is to be promoted.
      null if no promotion.
  */
  private void movePiece(BoardTile from,BoardTile target,Piece promoteTo) {
    StringBuffer strb=new StringBuffer(16);
    boolean enPassant=(from.getPiece().getType() == Piece.PAWN &&
		       target.xpos != from.xpos &&
		       target.getPiece() == null);

    if(enPassant) {
      grid.setPiece(target.xpos,from.ypos,null);
    }

    if(from.getPiece().getTypeChar()!=0)
      strb.append(from.getPiece().getTypeChar());

    if(target.getPiece()!=null || enPassant) {
      if(from.getPiece().getType()==Piece.PAWN)
	strb.append(from.getColumnCode());
      strb.append('x');
    }
    strb.append(target);

    // we have to make the move and promote before we can determine check.
    prevTarget=target;

    if(promoteTo==null)
      grid.setPiece(target.xpos,target.ypos,from.getPiece());
    else
      grid.setPiece(target.xpos,target.ypos,promoteTo);

    grid.setPiece(from.xpos,from.ypos,null);


    //are we promoting?

    if(promoteTo!=null && promoteTo.getType()!=Piece.PAWN)
      strb.append(promoteTo.getTypeChar());

    //do we have a check?
    int otherSide=(target.getPiece().getSide()==Piece.WHITE?
		   Piece.BLACK:Piece.WHITE);
    if(isCheckPos(otherSide,grid.kingTiles(otherSide))!=null){
      strb.append('+');
      if(checkForMate(otherSide)) {
	strb.append('+');
	gameOver(whosTurn);
      }
    } else
      checkBlockers=null;

    if(enPassant)
      strb.append(" e.p.");

    if(target.getPiece().getType()==Piece.KING) {
      if((target.xpos-from.xpos)==2)
	strb=new StringBuffer(16).append("o-o");
      else if((target.xpos-from.xpos)==-2)
	strb=new StringBuffer(16).append("o-o-o");
    }
    
    ctrl.addMove(whosTurn,strb);

    if(whosTurn<Piece.GAMEOVER && !canSideMove(otherSide)) {
     gameOver(Piece.DRAW);
    }


    if(whosTurn>=Piece.GAMEOVER)
      recordGameOver(whosTurn);
  }


  /** Adds result of the game to movelist, if game is over.
      @param winner Side of the winner.
   */
  private void recordGameOver(int winner) {
    StringBuffer strb;


    if(winner%Piece.GAMEOVER==Piece.DRAW) {
      strb=new StringBuffer(16).append(CtrlPanel.DRAW);
      ctrl.addMove(Piece.WHITE,strb);
      ctrl.addMove(Piece.BLACK,strb);
    } else {
      strb=new StringBuffer(16).append(CtrlPanel.VICTORY);
      ctrl.addMove(winner%Piece.GAMEOVER,strb);
    } 
  }

  /** Checks if specified player can move any pieces.
      @param who Id of the side who we are looking for.
      @return true if who can move.
  */
  boolean canSideMove(int who) {
    int rows[][]={{1,0,2,3,4,5,6,7},
		  {6,7,5,4,3,2,1,0}};
    BoardTile tile;

    for(int r=0;r<rows[0].length;r++) {
      for(int c=0;c<ChessSettings.COLUMNS;c++) {
	tile=grid.getTile(c,rows[who][r]);

	try {
	  if(tile.getPiece().getSide()==who && canMove(tile))
	    return true;
	} catch(NullPointerException e) {}
      }
    }
    return false;
  }

  /** Pause the timers. 
   */
  void pause() { 
    processClicks=false;
    ctrl.pause();
  }
  /** Unpause the timers. This will start the white's timer if the window
      is iconified and deiconified before first move. (good or bad??)
   */
  void unpause() {
    if(whosTurn<Piece.GAMEOVER) {
      processClicks=true;
      ctrl.unpause(whosTurn);
    }
  }

  /** Sets the game to game over-state.
      @param winner Winner of the game or Piece.DRAW
   */
  private void gameOver(int winner) {
    processClicks=false;
    whosTurn=winner+Piece.GAMEOVER;
    ctrl.killAll();
  }

  /* Chess Rules part */

  /** Checks if a piece in specific tile can move. Note: tile must
      contain a piece.
      @param tile Tile on the board, containing a piece.
  */
  private boolean canMove(BoardTile tile) {
    boolean result;
    BoardTile threat=null;

    if(tile.getPiece()==null)
      return false;

    try {
      threat=blocksThreatAt(tile.getPiece().getSide(),tile);
    } catch(NullPointerException e) {
      System.err.println("Serious trouble in canMove():"+e+
			 "\nThe tile is "+tile);
    }

    switch(tile.getPiece().getType()) {
    case Piece.PAWN:
      result=canPawnMove(tile);

      /* If piece is blocking a threat, it eather can move to threat
	 or it can't move at all.
      */
      if(result && threat!=null)
	result=validPawnMove(tile,threat);
      break;
    case Piece.ROOK:
      result=canRookMove(tile);

      if(result && threat!=null)
	result=validRookMove(tile,threat);
      break;
    case Piece.BISHOP:
      result=canBishopMove(tile);

      if(result && threat!=null)
	result=validBishopMove(tile,threat);
      break;
    case Piece.QUEEN:
      result=canQueenMove(tile);

      if(result && threat!=null)
	result=validQueenMove(tile,threat);
      break;
    case Piece.KNIGHT:
      result=canKnightMove(tile);

      if(result && threat!=null)
	result=false; //It's just imposible for knight
      break;
    case Piece.KING:
      result=canKingMove(tile);
      break;
    default:
      System.err.println("Unknown piece. Weird!");
      result=false;
    }
    return result;
  }

  /** This checks if bishop,knight,rook or queen can move.
      @param tile Tile where the piece is.
      @param side Side of the moving piece.
      @param cords The coordinates relative to tile, to where the piece can move.
      @see Piece.bishopMoves
      @see Piece.knightMoves
      @see Piece.rookMoves
  */
  private boolean canSimpleMove(BoardTile tile, int side, int cords[][]) {
   Piece tmp;
   
    for(int i=0;i<cords.length;i++) {
      try {
	tmp=grid.getPiece((tile.xpos+cords[i][0]),
			  (tile.ypos+cords[i][1]));
	if(tmp==null || tmp.getSide()!=side)
	  return true;
      } catch(ArrayIndexOutOfBoundsException e) { }
    }

    return false;
  }

  /** Checks if king on the tile can move.
      @param tile The tile where the knight is.
      @return true if can move, false if not.
  */
  private boolean canKingMove(BoardTile tile) {
    //if king can fortify, it can also move to some of these.
    int kingMoves[][]={{-1,1},{0,1},{1,1},
		       {-1,0}, /**/ {1,0},
		       {-1,-1},{0,-1},{1,-1}};
    BoardTile moveTo;

    for(int i=0;i<kingMoves.length;i++) {
      try {
	moveTo=grid.getTile((tile.xpos+kingMoves[i][0]),
			    (tile.ypos+kingMoves[i][1]));

	if(validKingMove(tile,moveTo))
	  return true;
      } catch(ArrayIndexOutOfBoundsException e) {}
    }
    return false;
  }

  /** Checks if knight on the tile can move.
      @param tile The tile where the knight is.
      @return true if can move, false if not.
  */
  boolean canKnightMove(BoardTile tile) {
    return canSimpleMove(tile,tile.getPiece().getSide(),Piece.knightMoves);
  }

  /** Checks if rook on the tile can move.
      @param tile The tile where the rook is.
      @return true if can move, false if not.
  */
  boolean canRookMove(BoardTile tile) {
    return canSimpleMove(tile,tile.getPiece().getSide(),Piece.rookMoves);
  }
  /** Checks if queen on the tile can move.
      @param tile The tile where the queen is.
      @return true if can move, false if not.
  */
  boolean canQueenMove(BoardTile tile) {
    return canBishopMove(tile)||canRookMove(tile);
  }
  

  /** Checks if pawn on the tile can move.
      @param tile The tile where the pawn is.
      @return true if can move, false if not.
  */
  boolean canPawnMove(BoardTile tile) {
    int newRow=(tile.getPiece().getSide()==Piece.WHITE?1:-1)+tile.ypos;


    /* If the tiles that are tested are non-existent or null, this
       throws exception. */
    try {
    if(grid.getPiece(tile.xpos,newRow)==null)
      return true;
    } catch(ArrayIndexOutOfBoundsException e) {}
    try {
      if(grid.getPiece(tile.xpos+1,newRow).getSide() != 
	 tile.getPiece().getSide())
	return true;
    } catch(Exception e) {}
    try {
      if(grid.getPiece(tile.xpos-1,newRow).getSide() != 
	 tile.getPiece().getSide())
	return true;
    } catch(Exception e) {}

    // Can we en passant??
    if(prevTarget.getPiece().getType()==Piece.PAWN) {
      try {
	if(prevTarget.getPiece()==grid.getPiece(tile.xpos-1,tile.ypos) &&
	   grid.getPiece(tile.xpos-1,newRow)==null)
	  return true;
      } catch(ArrayIndexOutOfBoundsException e) {}
      try {
	if(prevTarget.getPiece()==grid.getPiece(tile.xpos+1,tile.ypos) &&
	   grid.getPiece(tile.xpos+1,newRow)==null)
	  return true;
      } catch(ArrayIndexOutOfBoundsException e) {}
    }
    return false;
  }

  /** Checks if bishop on the tile can move.
      @param tile The tile where the bishop is.
      @return true if can move, false if not.
  */
  boolean canBishopMove(BoardTile tile) {
    return canSimpleMove(tile,tile.getPiece().getSide(),Piece.bishopMoves);
  }

}
