package kpalin.alabra;
import java.awt.Dimension;
import java.awt.Font;
import java.io.Serializable;

/** This is container for initialization information.

  @author Kimmo Palin
*/
public class ChessSettings implements Serializable{
  /** Number of rows on a chessboard
    */
  final static int ROWS=8;
  /** Number of columns on a chessboard
    */
  final static int COLUMNS=8;
  /** Dimensions of a tile.
   */
  private static Dimension tileDim=new Dimension(32,32);

  /** Wheather fortification is possible.
      @see ChessPlay
  */
  private boolean canFortify[][]={{true,true},{true,true}};

  /** Memory representation of the chessboard. Pieces are in their positions
      and nulls are in empty spaces. A1 is at (0,0) and A2 at (1,0)
  */
  private Piece [][] board;

  /** Side which takes first move
   */
  private int startSide=Piece.WHITE;

  /** Name of the font to be used in Control Panel.
    */
  private String fontName="Serif";

  /** Size of the font to be used in Control Panel.
    */
  private int fontSize=24;

  /** Time limit, in which must moveLimit moves be made.
   */
  private int timeLimit=0;

  /** Number of moves that must be made in timeLimit. If moveLimit =0
      all moves must be made in timeLimit.
   */
  private int moveLimit=0;

  /** Sets new time limit to time or 0 if time<0.
      @param time New time limit.
  */
  public void setTimeLimit(int time) {
    if(time>0)
      timeLimit=time;
    else
      timeLimit=0;
  }

  /** @return The time limit.
   */
  public int getTimeLimit() { return timeLimit; }

  /** Sets move limit to moves, or 0 if moves<0.
      @param moves The new limit.
  */
  public void setMoveLimit(int moves) {
    if(moves>0)
      moveLimit=moves;
    else
      moveLimit=0;
  }
  /** @return Move limit.
   */
  public int getMoveLimit() {return moveLimit; }

  /** Sets quickchess rules. All moves in 5 minutes.
   */
  public void setQuickChess() {
    setMoveLimit(0);
    setTimeLimit(5*60);
  }

  /** Set new name for Control Panel font.
    */
  public void setFontName(String name) {
    fontName=name;
  }
  /** Set new size for Control Panel font.
    */
  public void setFontSize(int size) {
    if(size>4)
      fontSize=size;
  }

  /** Returns the font to be used in Control Panel. Doesn't (can't) check
      for validity of the parameters.
  */
  public Font getFont() {
    if(fontSize<2)
      fontSize=24;
    return new Font(fontName,Font.PLAIN,fontSize);
  }

  /** Creates new ChessSettings. That is initializes an empty board.
    */
  public ChessSettings() {
    board=new Piece[ROWS][COLUMNS];
  }

  /** Sets the dimensions of board tiles.
      @param d The tile dimension.
  */
  static void setTileDim(Dimension d) {
    if(d.height<5)
      d.height=32;
    if(d.width<5)
      d.width=32;

    tileDim=d;
  }
  /** Gives the tile dimension.
      @return The tile dimension.
  */
  static Dimension getTileDim() { return tileDim; }
   
  void setFortify(int side, int type, boolean value) {
    if((side==Piece.WHITE || side==Piece.BLACK) &&
       (type==Piece.LONGFORT || type==Piece.SHORTFORT))
    canFortify[side][type]=value;
  }

  boolean[][] fortClone() {
    boolean [][] cln=new boolean[canFortify.length][canFortify[0].length];

    for(int i=0;i<canFortify.length;i++){
      for(int j=0;j<canFortify[i].length;j++) {
	cln[i][j]=canFortify[i][j];
      }
    }
    return cln;
  }

  /** Returns a Piece at specified position of the board.
      @param row Row of the piece
      @param col Column of the piece
      @return Piece in position (row,col)
      @exception ArrayIndexOutOfBoundsException Tried to reach inexistent location.
  */
  Piece pieceAt(int row,int col) {
    return board[row][col];
  }

  /** Sets a given Piece to a given location.
      @param row Row where to put the piece.
      @param col Column where to put the piece.
      @param p The piece to be put.;) Can also be null. (remove piece)
      @exception ArrayIndexOutOfBoundsException User of the class can
      do with this what ever they like.
      */
  void setPieceTo(int row,int col,Piece p) throws ArrayIndexOutOfBoundsException {
    board[row][col]=p;
  }
  /** Sets the beginning side.
      @param side The starting side (Piece.WHITE or Piece.BLACK)
  */
  void setStartSide(int side) {
    if(side==Piece.WHITE && side==Piece.BLACK)
      startSide=side;
    else
      startSide=Piece.WHITE;
  }
  /** Gives the starting side.
      @return The starting side.
  */
  int getStartSide(){
    return startSide;
  }
  
  /** Resets the board to normal initial standing.
    */
  void initStanding() {
    board=new Piece[ROWS][COLUMNS];
    startSide=Piece.WHITE;
    int officerStand[]={Piece.ROOK,Piece.KNIGHT,Piece.BISHOP,Piece.QUEEN,
			Piece.KING,Piece.BISHOP,Piece.KNIGHT,Piece.ROOK};

    try {
    for(int c=0;c<COLUMNS;c++) {
      board[0][c]=new Piece(Piece.WHITE,officerStand[c]);
      board[1][c]=new Piece(Piece.WHITE,Piece.PAWN);
      board[6][c]=new Piece(Piece.BLACK,Piece.PAWN);
      board[7][c]=new Piece(Piece.BLACK,officerStand[c]);
    }
    }catch(Exception e) { System.err.println(e); }
  }
}


