package kpalin.alabra;
import java.awt.*;
import java.awt.event.*;


/** Other major panel on the screen. Contains timers, reset and 
    historylist.

    @author Kimmo Palin
*/

public class CtrlPanel extends Panel {
  /** List to show move history.
   */
  private List history[];
  public static final String DRAW="Draw!";
  public static final String VICTORY="Victory!";

  /** The panel holding timers and reset-button.
   */
  private Panel top;
  
  /** These are the timers for black and white. The indexies are same
      as in Piece-class.
  */
  private ChessTimer timers[];

  /** Number of moves before next timeslize.
   */
  private int moveLimit=0;

  /** The reset-button.
   */
  private Button rsetButton;

  /** Creates new controls.
      @param blisten Listener for button action.
      @param moves Number of moves in timeShare
      @param timeShare Time in which must moves be made.
   */
  CtrlPanel(ChessCtrlCallback blisten,int moves,int timeShare) {
    setLayout(new GridLayout(2,1));

    top=initTop(blisten,timeShare);
    Panel bottom=initBottom();

    setMoveLimit(moves);

    add(top);
    add(bottom);
  }

  private Panel initBottom() {
    Panel bott=new Panel();

    history=new List[2];
    history[0]=new List();
    history[1]=new List();

    bott.setLayout(new GridLayout(1,2,30,0));
    bott.add(history[0]);
    bott.add(history[1]);

    return bott;
  }

  private void setMoveLimit(int moves) {
    if(moves>0)
      moveLimit=moves;
    else
      moveLimit=0;
  }

  /** Adds a timeshare to given player if he has reached given
      movecount.
      @param side Side of the player.
      @param count Number of moves player has made.
  */
  private void addShare(int side,int count) {
    if(moveLimit!=0 && count%moveLimit==0) {
      timers[side%Piece.GAMEOVER].newTimeSlize();
    }
  }

  /** Resets the control information.
    */
  void resetAll() {
    for(int i=0;i<2;i++) {
      history[i].removeAll();
      timers[i].reset();
    }
  }

  /** Kills timers. Possibly current thread last.
    */
  public void killAll() {
    ChessTimer thisLast=null;

    for(int i=0;i<2;i++) {
      if(!timers[i].isCurrentThread())
	timers[i].stop();
      else
	thisLast=timers[i];
    }
    if(thisLast!=null)
      thisLast.stop();
  }    
  /** Pauses both timers. This has one rather nasty feature: according to
      netscape, calling Thread.suspend() might result navigator deadlock.
   */
  void pause() {
    for(int i=0;i<2;i++)
      timers[i].suspend();
  }

  /** This unpauses the timers. Resumes them only if there has been
      recorded moves.
      @param who Side whose turn it is.
  */
  void unpause(int who) {
    if(history[0].getItemCount()>0 || history[1].getItemCount()>0)
      resumeTimer(who);
  }

  /** Resumes the timer for specified side. Stops the other players timer. 
      This has one rather nasty feature: according to netscape, calling 
      Thread.suspend() might result navigator deadlock.
      @param side Side whose timer is to be resumed.
   */
  void resumeTimer(int side) {
    try{
      if(side==Piece.WHITE)
	timers[Piece.BLACK].suspend();
      else
	timers[Piece.WHITE].suspend();
      
      timers[side].resume();
    } catch(Exception e) { System.out.println(e); }
  }

  /** This adds a half-move to historylist.
      @param side The movers side (Piece.WHITE,Piece.BLACK)
      @param strb String representation of the move.
  */
  void addMove(int side,StringBuffer strb) {
    side=side%Piece.GAMEOVER;

    if(side==Piece.DRAW) {
      // This is a hack to make copy of StringBuffer
      StringBuffer tmp=new StringBuffer().append(strb);
      addMove(Piece.WHITE,tmp);
      addMove(Piece.BLACK,strb);
      return;
    } //else

    strb.append(' ').append(timers[side]);

    // According to api, this is useless but they're just kidding.
    try {
      history[side].deselect(history[side].getSelectedIndex());
    } catch(Exception e) {}
      
    history[side].add(strb.toString());

    int count=history[side].getItemCount();

    addShare(side,count);

    for(int j=0;j<2;j++) { 
      history[j].makeVisible(count-1);
    }
  }

  /** Finds out number of made moves.
      @param side Side whose movecount we want.

      @return Number of made moves.
  */
  int noOfMoves(int side) {
    return history[side%Piece.GAMEOVER].getItemCount();
  }

  /** This initializes the top-panel. This is just to clean up the
      constructor.
      @param blisten Button listener
  */
  private Panel initTop(ChessCtrlCallback blisten,int timeShare) {
    Panel futuretop=new Panel();
    futuretop.setLayout(new GridLayout(3,1));

    timers=new ChessTimer[2];
    timers[Piece.WHITE]=new ChessTimer("White",Piece.WHITE,timeShare,blisten);
    timers[Piece.BLACK]=new ChessTimer("Black",Piece.BLACK,timeShare,blisten);

    rsetButton=new Button("Reset");
    rsetButton.addActionListener(blisten);

    futuretop.add(timers[Piece.BLACK]);
    futuretop.add(timers[Piece.WHITE]);
    futuretop.add(rsetButton);
    
    return futuretop;
  }

  public int[] getTimes() {
    int times[]=new int[2];

    for(int i=0;i<2; i++) 
      times[i]=timers[i].getTime();
    return times;
  }

  /** We don't allow to change font in historylists.
    */
  public void setFont(Font f) {
    // The stock setFont() changes only appropriate values.
    top.setFont(f);
  }

}




