package kpalin.alabra;
import java.awt.image.*;
import java.awt.*;
import java.io.*;

/** This is a datastructure representing the chess pieces. Also the images
    of the pieces are generated and stored here as static data.
    @see PieceImageMemData
    @see ChessSettings
    @author Kimmo Palin
 */
public class Piece implements Serializable{

  /** Errormessage body
    */
  final static String ERRMESG="Wrong piece type or side!!";

  /** These are the relative coordinates where rook can move.
   */
  final static int rookMoves[][]={{0,1},{1,0},{-1,0},{0,-1}};

  /** These are the relative coordinates where bishop can move.
   */
  final static int bishopMoves[][]={{1,1},{1,-1},{-1,1},{-1,-1}};

  /** These are the relative coordinates where knight can move.
   */
  final static int knightMoves[][]={{-1,2},{1,2},
				    {-2,1},{2,1},
				    {-2,-1},{2,-1},
				    {-1,-2},{1,-2}};

  /** Images of all pieces. Just stored here. Use accessories to add
      images. Can't make this final 'cause of initialization difficulties. 
   */
  private static Image PIECES[][];

  /** These are the names of the gif-files that hold the pieceimages.
   */
  public static final String IMAGENAME[][]={
    {"k64o.gif","q64o.gif","r64o.gif","b64o.gif","n64o.gif","p64o.gif"},
    {"k64s.gif","q64s.gif","r64s.gif","b64s.gif","n64s.gif","p64s.gif"}};

  /** These are the characters to identify the pieces
    */
  static final char TYPECHARS[]={'K','Q','R','B','N',0};

  /** Indexies for pieces
   */
  final static int WHITE=0;
  final static int BLACK=1;
  /** This is added to turn-variable so that WHITE+GAMEOVER is white victory
      and BLACK+GAMEOVER is black victory.
  */
  final static int GAMEOVER=5;
  final static int DRAW=3;

  final static int KING=0;
  final static int QUEEN=1;
  final static int ROOK=2;
  final static int BISHOP=3;
  final static int KNIGHT=4;
  final static int PAWN=5;

  final static int LONGFORT=0;
  final static int SHORTFORT=1;

  /** This is ment for Applet and Application to add the images upon
      initialization.
      @param s Side.
      @param t Type.
      @param i Image of the piece.
  */
  public static void addImage(int s,int t, Image i) {
    if(PIECES==null)
      PIECES=new Image[BLACK+1][PAWN+1];
    PIECES[s][t]=i;
  }
  /** This gives the white pawn (used as an icon in application).
      Throws ArrayIndexOutOfBoundsException if PIECES is not
      initialized.
   */
  static Image getIcon() {
    return PIECES[WHITE][PAWN];
  }

  /** Side of the piece. White or black.
   */
  private int side;
  /** Type of the piece. See final static fields.
   */
  private int type;

  /** Writing Piece-object to ObjectOutput.
   */
  public void writeExternal(ObjectOutput out) throws IOException {
    out.write(side);
    out.write(type);
  }
  /** Reading Piece from ObjectInput.
   */
  public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException{
    side=in.read();
    type=in.read();
  }

  /** Creates a new piece, with specified type and side. If the parameters
      are not valid, throws exception.

      @param bw Side of the piece. Black or white.
      @param piece Type of the piece.
      @exception Exception Error if you want to do inexistent piece.
    */
  Piece(int bw,int piece) throws Exception {
    
    if(bw==BLACK || bw==WHITE)
      side=bw;
    else
      this.side=-1;
    if(piece>=KING && piece<=PAWN)
      this.type=piece;
    else
      this.type=-1;
    
    if(this.side<0 || this.type<0)
      throw new Exception(ERRMESG);
  }

  /** Query type of the piece.
      @return Type of the piece.
  */
  int getType() { return type; }

  /** Query type of the piece.
      @return Character representing the type of this piece.
  */
  char getTypeChar() {
    return TYPECHARS[type];
  }

  /** Query side of the piece. Black or white.
      @return Side of the piece.
  */
  int getSide() { return side; }

  /** Get image of the piece.
      @return Image of the piece.
  */
  Image getImage() { return PIECES[side][type]; }

  public String toString() {
    return (getSide()==BLACK?"Black ":"White ")+getType();
  }
}
