package kpalin.alabra;
import java.awt.*;
import java.awt.event.*;

/** This is the key class of the whole program. This is attached on
    application window or an applet panel and the whole chess stuff is
    under this class.

    @author Kimmo Palin
*/

public class ChessPlay extends Panel implements MouseListener,ActionListener{
  /** The panel that holds the chess board.
   */
  private BoardGrid grid;

  /** Initial settings got with an implementation specific way.
   */
  private final ChessSettings initSet;

  /** The panel that holds all control etc. stuff on the screen.
   */
  private CtrlPanel ctrl;

  /** Process click-events. Turned of during pawn promotion and
      after a mate.
   */
  private boolean processClicks=true;
  /** Whos turn is it?.
      @see Piece.WHITE
      @see Piece.BLACK
   */
  private int whosTurn;

  /** The tile where is the piece to be moved. Not really good place
      for this stuff but can't help it. Only mouseClicked can use this!
      @see mouseClicked
   */
  private BoardTile startTile;

  /** The previous move. This is needed for the en'passant.
   */
  private BoardTile prevTarget;

  /** Possibility of fortification. Left dimension is side, right 
      Piece.LONGFORT or Piece.SHORTFORT
  */
  private boolean canFortify[][]={{true,true},{true,true}};

  /** Promotion windows. No need to create these every time.
   */
  PawnPromote pPromote[];

  /** Creates new chess play object.
   */
  ChessPlay(ChessSettings set) {
    initSet=set;

    setLayout(new GridLayout(1,2));
    grid=new BoardGrid(this);

    ctrl=new CtrlPanel(this);

    grid.setBoard(initSet);
    whosTurn=initSet.getStartSide();

    add(grid);
    add(ctrl);

    pPromote=new PawnPromote[2];
    for(int i=0;i<2;i++)
      pPromote[i]=new PawnPromote(i);
  }


  /* Event listening part. */

  /** Reset the game*/
  public void resetGame() {
    grid.setVisible(false);
    grid.setBoard(initSet);
    ctrl.resetAll();
    try {
      startTile.setSelected(false);
      startTile=null;
    } catch(NullPointerException e) {}
    

    canFortify=initSet.fortClone();
    whosTurn=initSet.getStartSide();
    grid.setVisible(true);
  }


  /** Listens for the reset button on ctrlPanel.
   */
  public void actionPerformed(ActionEvent e) {
    System.err.println("reset:"+e);
    resetGame();
  }
  public void mouseEntered(MouseEvent e) { return; }
  public void mouseExited(MouseEvent e) { return; }
  public void mousePressed(MouseEvent e) { return; }
  public void mouseReleased(MouseEvent e) { return; }

  /** This is the callback method for the BoardTiles. This is the best
      place for this because it needs connections to ctrl (change turn)
      and grid (information about the board status)
   */
  public void mouseClicked(MouseEvent e) {
    BoardTile source=(BoardTile)e.getComponent();

    System.out.println(source);

    if(startTile!=null) {
      /* this is the second click of the move. (the target tile)*/
      System.err.println("Move completion!");
      makeMove(startTile,source);
    } else if(!processClicks || 
	      source.getPiece()==null || 
	      whosTurn!=source.getPiece().getSide() ||
	      !canMove(source)) {
      return; /* somehow invalid tile*/
    } else {
      System.out.println(source.getPiece());
      startTile=source;
      source.setSelected();
    }
  }

  /** Takes care of the boring routines when we change turn.
   */
  private void changeTurn() {
    startTile.setSelected(false); 
    startTile=null;


    if(whosTurn==Piece.WHITE)
      whosTurn=Piece.BLACK;
    else
      whosTurn=Piece.WHITE;

    if(checkForMate(whosTurn)) {
      processClicks=false;
      ctrl.pause();
    } else
      ctrl.resumeTimer(whosTurn);
  }


  /** Validate and make the move.
      @param from The tile where the move starts.
      @param target The target tile where to move the piece.
  */
  private void makeMove(BoardTile from,BoardTile target) {
    if(target==from)
      return;
    System.out.println("Trying to move "+from.getPiece()+
		       " from " +from+" to " +target);

    BoardTile threat=blocksThreatFrom(from.getPiece().getSide(),from);

    //doesn't eliminate the threat
    if(threat!=null && threat!=target){
      BoardTile newThreat=blocksThreatFrom(from.getPiece().getSide(),target);

       //doesn't block a threat in same direction anymore
      if(newThreat==null ||
	 (threat.xpos-from.xpos) != (newThreat.xpos-target.xpos) || 
	 (threat.ypos-from.ypos) != (newThreat.ypos-target.ypos)) {
	System.out.println("Trying to give in check. (towards "+threat+")");
	return;
      }
    }
  
    switch(from.getPiece().getType()) {
    case Piece.PAWN:
      if(!validPawnMove(from,target)) 
	return;
      if((from.getPiece().getSide()==Piece.WHITE && target.ypos==7) ||
	 (from.getPiece().getSide()==Piece.BLACK && target.ypos==0)){
	System.err.println("Pawn promotion");
	pPromote[from.getPiece().getSide()].show();
	
      }
      break;
    case Piece.ROOK:
      if(from.xpos==0 && // Check that rook hasn't moved
	 from.ypos==grid.kingTiles(from.getPiece().getSide()).ypos)
	canFortify[from.getPiece().getSide()][Piece.LONGFORT]=false;
      else
	canFortify[from.getPiece().getSide()][Piece.SHORTFORT]=false;

      if(!validRookMove(from,target))
	return;
      break;
    case Piece.BISHOP:
      if(!validBishopMove(from,target))
	return;
      break;
    case Piece.QUEEN:
      if(!validQueenMove(from,target))
	return;
      break;
    case Piece.KNIGHT:
      if(!validKnightMove(from,target)) 
	return;
      break;
    case Piece.KING:
      if(validFort(from,target)) {
	int rookFrom=0,rookTo=3;
	if(from.xpos<target.xpos) { //SHORTFORT
	  rookFrom=7;
	  rookTo=5;
	}
	grid.setPiece(rookTo,target.ypos,grid.getPiece(rookFrom,target.ypos));
	grid.setPiece(rookFrom,target.ypos,null);

      } else if(!validKingMove(from,target)) {
	return;
      }
    
      canFortify[from.getPiece().getSide()][Piece.LONGFORT]=false;
      canFortify[from.getPiece().getSide()][Piece.SHORTFORT]=false;

      break;
    default:
      System.out.println("jokumuu siirto");
      return;
    }


    movePiece(from,target);

    
    changeTurn();
    
  }

  /** Tries to clean up after stoping applet.
   */
  public void killAll() {
    ctrl.killAll();
  }
  /** An assistant method to validate moves. Analyses how a piece in a
      specific tile blocks threats to king. The return tile is always
      in (1,1)-(-1,-1) box near the given tile.

      @param side The moving side.
      @param tile The tile from where we are trying to move. 
      No need to have piece in here.
      @return null if piece in tile doesn't block any threats, otherwise
      the tile in the direction of the threat.
  */
  private BoardTile blocksThreatFrom(int side,BoardTile tile) {
    BoardTile king=grid.kingTiles(side);
    BoardTile threatTile=null;
    int deltax;
    int deltay;

    if(king.xpos==tile.xpos || king.ypos==tile.ypos || // maybe blocking.
       Math.abs(king.xpos-tile.xpos)==Math.abs(king.ypos-tile.ypos)) {

      try { //The threat comes from direction (deltax,deltay)
	deltax=(tile.xpos-king.xpos)/Math.abs(tile.xpos-king.xpos);
      } catch(ArithmeticException e) { deltax=0; }
      try {
	deltay=(tile.ypos-king.ypos)/Math.abs(tile.ypos-king.ypos);
      } catch(ArithmeticException e) { deltay=0; }
      
      // Trying to block with king?!?
      if(deltax==0 && deltay==0)
	return null;

      try {  // huge try-clause for ArrayIndexOutOfBoundsException
	// We start from king towards the requested tile so we can be sure
	// that tile is the only-one blocking the threat.
	int len=1;
	for(int jumpPiece=0;jumpPiece<2;jumpPiece++) {
	  while(grid.getPiece(king.xpos+deltax*len,king.ypos+deltay*len)==null)
	      len++;
	  
	  threatTile=grid.getTile(king.xpos+deltax*len,
				  king.ypos+deltay*len);

	  // if we don't reach tile first, it is not blocking.
	  System.err.println("jumpPiece:"+jumpPiece+":"+threatTile);
	  if(jumpPiece==0 && threatTile!=tile) 
	    return null;
	  len++; //jump over the blocker.
	}


	// Is there anything to be afraid of?
	if(threatTile.getPiece().getSide() != side &&
	   (threatTile.getPiece().getType() == Piece.QUEEN ||
	    (threatTile.getPiece().getType() == Piece.ROOK && 
	     (deltax==0|| deltay==0) ) ||
	    (threatTile.getPiece().getType() == Piece.BISHOP && 
	     (deltax!=0 && deltay!=0) ) ) ) {
	  return grid.getTile(tile.xpos+deltax,tile.ypos+deltay);
	}
      } catch(ArrayIndexOutOfBoundsException e) { }
    }
    return null;
			      
  }
  

  /** Checks validity of en'passant-move
      @param from Tile where the move starts. Here must be a piece.
      @param target Tile where player is moving a piece.
  */
  private boolean validEnPassant(BoardTile from,BoardTile target) {

    try {
      System.out.println(prevTarget);
      if(from.getPiece().getType()!=Piece.PAWN ||
	 prevTarget.getPiece().getType()!=Piece.PAWN)
	return false; //Both must be pawns.
    } catch(NullPointerException e) { 
      return false;
    } // The first move throws exception

    System.err.println("Pawns!");

    if(Math.abs(target.xpos-from.xpos)!=1 ||
       target.xpos!=prevTarget.xpos)
      return false; //Columns must match
    System.err.println("Columns!");
    
    if(from.getPiece().getSide()==Piece.WHITE &&
	(from.ypos!=4 || prevTarget.ypos!=4 ||
	 target.ypos!=5))
      return false;  // Row stuff for white
    System.err.println("Rows White!");

    if(from.getPiece().getSide()==Piece.BLACK &&
	(from.ypos!=3 || prevTarget.ypos!=3 ||
	 target.ypos!=2))
      return false;  // Row stuff for black

    System.err.println("Rows black");

    if(from.getPiece().getSide()==prevTarget.getPiece().getSide())
      System.err.println("We're totaly screwed, double move for one side!");

    return true;
  }

  /** Checks the validity of a move for a Pawn.
      @param target The target tile.
  */
  private boolean validPawnMove(BoardTile from,BoardTile target) {

    if(validEnPassant(from, target))
       return true;
    

    if(((from.ypos+1)==target.ypos && //WHITE going forward
	from.getPiece().getSide()==Piece.WHITE) || 
       ((from.ypos-1)==target.ypos && //BLACK going forward
	from.getPiece().getSide()==Piece.BLACK))  {

      //Straight
      if(from.xpos==target.xpos && target.getPiece()==null)
	return true;
      //Eating
      else if(Math.abs(from.xpos-target.xpos)==1 && 
	      target.getPiece()!=null && target.getPiece().getSide()!=
	      from.getPiece().getSide()) 
	return true;
    }

    else if(target.xpos==from.xpos && 
	    Math.abs(target.ypos-from.ypos)==2 &&
	    ((target.ypos == 3 && //WHITE going forward 2 steps
	      from.getPiece().getSide()==Piece.WHITE) || 
	     (target.ypos == 4 && //BLACK going forward 2 steps
	      from.getPiece().getSide()==Piece.BLACK))) 
      return true;

    return false;
  }

  /** Checks the validity of a move for a Bishop.
      @param target The target tile.
  */
  private boolean validBishopMove(BoardTile from,BoardTile target) {
    if(Math.abs(target.xpos-from.xpos)!=
       Math.abs(target.ypos-from.ypos))
      return false;

    int delta=Math.abs(target.xpos-from.xpos);

    if(delta==0)  // We'll never reach here but in any case..
      return false;

    int signx=(target.xpos-from.xpos)/delta;
    int signy=(target.ypos-from.ypos)/delta;


    try {  // Check that we don't eat own piece
      if(target.getPiece().getSide()==from.getPiece().getSide())
	return false;
    } catch(NullPointerException e) {}

    return validStraightMove(from,signx,signy,delta);
  }

  /** Checks for threatening knights.
      @param side The kings side. (Piece.WHITE or Piece.BLACK)
      @param kingPos The tile whose threat to analyse. (note: no need for a
      King to be in this tile.)
      @return The tile where is the threatening piece.
  */
  private BoardTile knightThreat(int side, BoardTile kingPos) {
    Piece tmp;

    /* Check for threatening knights*/
    for(int i=0;i<Piece.knightMoves.length;i++) {
      try {
	tmp=grid.getPiece(kingPos.xpos+Piece.knightMoves[i][0],
			  kingPos.ypos+Piece.knightMoves[i][1]);
	if(tmp.getType()==Piece.KNIGHT && tmp.getSide()!=side)
	  return grid.getTile(kingPos.xpos+1,kingPos.ypos+2);
      } catch(Exception e) {} //NullPointer or ArrayIndexOutOfBounds
    }/*
    try {
    tmp=grid.getPiece(kingPos.xpos-1,kingPos.ypos+2);
    if(tmp.getType()==Piece.KNIGHT && tmp.getSide()!=side)
      return grid.getTile(kingPos.xpos-1,kingPos.ypos+2);
    } catch(Exception e) {}
    try {
    tmp=grid.getPiece(kingPos.xpos+1,kingPos.ypos-2);
    if(tmp.getType()==Piece.KNIGHT && tmp.getSide()!=side)
      return grid.getTile(kingPos.xpos+1,kingPos.ypos-2);
    } catch(Exception e) {}
    try {
    tmp=grid.getPiece(kingPos.xpos-1,kingPos.ypos-2);
    if(tmp.getType()==Piece.KNIGHT && tmp.getSide()!=side)
      return grid.getTile(kingPos.xpos-1,kingPos.ypos-2);
    } catch(Exception e) {}
    try {
    tmp=grid.getPiece(kingPos.xpos+2,kingPos.ypos+1);
    if(tmp.getType()==Piece.KNIGHT && tmp.getSide()!=side)
      return grid.getTile(kingPos.xpos+2,kingPos.ypos+1);
    } catch(Exception e) {}
    try {
    tmp=grid.getPiece(kingPos.xpos-2,kingPos.ypos+1);
    if(tmp.getType()==Piece.KNIGHT && tmp.getSide()!=side)
      return grid.getTile(kingPos.xpos-2,kingPos.ypos+1);
    } catch(Exception e) {}
    try {
    tmp=grid.getPiece(kingPos.xpos+2,kingPos.ypos-1);
    if(tmp.getType()==Piece.KNIGHT && tmp.getSide()!=side)
      return grid.getTile(kingPos.xpos+2,kingPos.ypos-1);
    } catch(Exception e) {}
    try {
    tmp=grid.getPiece(kingPos.xpos-2,kingPos.ypos-1);
    if(tmp.getType()==Piece.KNIGHT && tmp.getSide()!=side)
      return grid.getTile(kingPos.xpos-2,kingPos.ypos-1);
    } catch(Exception e) {}
     */
    System.err.println("Kinght null");
    return null;
  }

  /** Checks for threatening pawns.
      @param side The kings side. (Piece.WHITE or Piece.BLACK)
      @param kingPos The tile whose threat to analyse. (note: no need for a
      King to be in this tile.)
      @return The tile where is the threatening piece.
  */
  private BoardTile pawnThreat(int side,BoardTile kingPos) {
    int pawnRow=(side==Piece.WHITE?1:-1) + kingPos.ypos;
    Piece tmp;

    try {
      tmp=grid.getPiece(kingPos.xpos-1,pawnRow);
      if(tmp.getSide()!=side && tmp.getType()==Piece.PAWN)
	return grid.getTile(kingPos.xpos-1,pawnRow);;
    } catch(Exception e) {}
    try {
      tmp=grid.getPiece(kingPos.xpos+1,pawnRow);
      if(tmp.getSide()!=side && tmp.getType()==Piece.PAWN)
	return grid.getTile(kingPos.xpos+1,pawnRow);
    } catch(Exception e) {}

    System.err.println("Pawn null");
    return null;
  }

  /** Checks for threatening rooks, bishops, and queen.
      @param side The kings side. (Piece.WHITE or Piece.BLACK)
      @param kingPos The tile whose threat to analyse. (note: no need for a
      King to be in this tile.)
      @return The tile where is the threatening piece.
  */
  private BoardTile otherThreat(int side, BoardTile kingPos) {
    Piece tmp;
    int i=-1;

    tmp=null;
    try {
      for(i=0;tmp==null;) {
	i++;
	tmp=grid.getPiece(kingPos.xpos+i,kingPos.ypos);
      } 
      if(tmp.getSide()!=side && (tmp.getType()==Piece.ROOK || 
				 tmp.getType()==Piece.QUEEN ||
				 // i increases after tmp being not null
				 (tmp.getType()==Piece.KING &&i==2)))
	return grid.getTile(kingPos.xpos+i-1,kingPos.ypos);
    } catch(ArrayIndexOutOfBoundsException e) {}

    tmp=null;
    try {
      for(i=0;tmp==null;) {
	i++;
	tmp=grid.getPiece(kingPos.xpos-i,kingPos.ypos);
      }
      if(tmp.getSide()!=side && (tmp.getType()==Piece.ROOK || 
				 tmp.getType()==Piece.QUEEN ||
				 (tmp.getType()==Piece.KING &&i==1)))
	return grid.getTile(kingPos.xpos-i,kingPos.ypos);;
    } catch(ArrayIndexOutOfBoundsException e) {}

    tmp=null;
    try {
      for(i=0;tmp==null;){
	i++;
	tmp=grid.getPiece(kingPos.xpos,kingPos.ypos+i);
      }if(tmp.getSide()!=side && (tmp.getType()==Piece.ROOK || 
				 tmp.getType()==Piece.QUEEN ||
				 (tmp.getType()==Piece.KING && i==1)))
	return grid.getTile(kingPos.xpos,kingPos.ypos+i);;
    } catch(ArrayIndexOutOfBoundsException e) {}

    tmp=null;
    try {
      for(i=0;tmp==null;) {
	i++;
	tmp=grid.getPiece(kingPos.xpos,kingPos.ypos-i);
      }
      if(tmp.getSide()!=side && (tmp.getType()==Piece.ROOK || 
				 tmp.getType()==Piece.QUEEN ||
				 (tmp.getType()==Piece.KING && i==1)))
	return grid.getTile(kingPos.xpos,kingPos.ypos-i);;
    } catch(ArrayIndexOutOfBoundsException e) {}



    tmp=null;
    try {
      for(i=0;tmp==null;) {
	i++;
	tmp=grid.getPiece(kingPos.xpos+i,kingPos.ypos+i);
      } if(tmp.getSide()!=side && (tmp.getType()==Piece.BISHOP || 
				 tmp.getType()==Piece.QUEEN ||
				 (tmp.getType()==Piece.KING && i==1)))
	return grid.getTile(kingPos.xpos+i,kingPos.ypos+i);;
    } catch(ArrayIndexOutOfBoundsException e) {}

    tmp=null;
    try {
      for(i=0;tmp==null;) {
	i++;
	tmp=grid.getPiece(kingPos.xpos-i,kingPos.ypos+i);
      }if(tmp.getSide()!=side && (tmp.getType()==Piece.BISHOP || 
				 tmp.getType()==Piece.QUEEN ||
				 (tmp.getType()==Piece.KING && i==1)))
	return grid.getTile(kingPos.xpos-i,kingPos.ypos+i);;
    } catch(ArrayIndexOutOfBoundsException e) {}

    tmp=null;
    try {
      for(i=0;tmp==null;) {
	i++;
	tmp=grid.getPiece(kingPos.xpos+i,kingPos.ypos-i);
      } if(tmp.getSide()!=side && (tmp.getType()==Piece.BISHOP || 
				 tmp.getType()==Piece.QUEEN ||
				 (tmp.getType()==Piece.KING && i==1)))
	return grid.getTile(kingPos.xpos+i,kingPos.ypos-i);;
    } catch(ArrayIndexOutOfBoundsException e) {}

      
    tmp=null;
    try {
      for(i=0;tmp==null;) {
	i++;
	tmp=grid.getPiece(kingPos.xpos-i,kingPos.ypos-i);
      } if(tmp.getSide()!=side && (tmp.getType()==Piece.BISHOP || 
				 tmp.getType()==Piece.QUEEN ||
				 (tmp.getType()==Piece.KING && i==1)))
	return grid.getTile(kingPos.xpos-i,kingPos.ypos-i);;
    } catch(ArrayIndexOutOfBoundsException e) {}

    System.err.println("Other null");

    return null;
  }

  /** Checks for checks.
      @param side The kings side. (Piece.WHITE or Piece.BLACK)
      @param kingPos The tile whose threat to analyse. (note: no need for a
      King to be in this tile.)
      @return The tile of the threat or null.
  */
  private BoardTile isCheckPos(int side, BoardTile kingPos) {
    System.out.println("KingPos:"+kingPos);
    BoardTile threat;

    threat=otherThreat(side,kingPos);
    if(threat!=null)
      return threat;

    threat=knightThreat(side,kingPos);
    if(threat!=null)
      return threat;

    threat=pawnThreat(side,kingPos);
    if(threat!=null)
      return threat;

    return null;

  }

  
  /** Checks the validity of a fortification.
      @param from Starting position of the king.
      @param target The target tile.
      @return true if can move, false if not
  */
  private boolean validFort(BoardTile from,BoardTile target) {
    int ourSide=from.getPiece().getSide();
    int kingsDelta=Math.abs(from.xpos-target.xpos);
    int route;

    if(from.getPiece().getType()!=Piece.KING)
      return false;
    if(from.ypos!=target.ypos)
      return false;
    if(kingsDelta!=2)
      return false;

    System.out.println("Beginning");
    // direction of kings movement.
    kingsDelta=(target.xpos-from.xpos)/kingsDelta;

    if((kingsDelta>0 && !canFortify[ourSide][Piece.SHORTFORT]) ||
       (kingsDelta<=0 && !canFortify[ourSide][Piece.LONGFORT]))
      return false;

    if(isCheckPos(ourSide,from)!=null)
      return false;
    System.err.println("Going strong");
    try {
    // No Pieces in between and no threats on first two tiles.
      for(route=1;
	  grid.getPiece(from.xpos+kingsDelta*route,target.ypos)==null &&
	    (route>2 || //Rook can move over threat.
	     isCheckPos(ourSide,grid.getTile(from.xpos+kingsDelta*route,
					     target.ypos))== null);route++ )
	{System.err.println(kingsDelta*route); }
    } catch(ArrayIndexOutOfBoundsException e) { return false; }		    

    try {
      if(grid.getPiece(from.xpos+kingsDelta*route,target.ypos).getType()
	 != Piece.ROOK)
	return false;
    }catch(NullPointerException e) {return false;}

    return true;
  }
  /** Checks the validity of a move for a King.
      @param from Starting position.
      @param target The target tile.
      @return true if can move, false if not
  */

  private boolean validKingMove(BoardTile from,BoardTile target) {

    if(Math.abs(target.xpos-from.xpos)>1 ||
       Math.abs(target.ypos-from.ypos)>1) 
      return false;
    try {
      if(target.getPiece().getSide()==from.getPiece().getSide())
	return false;
    }catch(NullPointerException e) {  }


    if(isCheckPos(from.getPiece().getSide(),target)!=null)
      return false;

    return true;
  }

  /** Checks the validity of a move for a Knight.
      @param target The target tile.
  */
  private boolean validKnightMove(BoardTile from,BoardTile target) {
    int dx=Math.abs(target.xpos-from.xpos);
    int dy=Math.abs(target.ypos-from.ypos);
    System.err.println("dy="+dy+" dx="+dx);

    try {
    return (((dy==1 && dx==2)||(dy==2&&dx==1)) && // correct location
	    (target.getPiece()==null || // empty tile     //or enemy piece
	    target.getPiece().getSide()!=from.getPiece().getSide()));
    } catch(NullPointerException e) {
      System.err.println("Trouble with validKnightMove: "+e);
      return false; 
    }

  }

  /** Checks the validity of a move for a Queen.
      @param target The target tile.
  */
  private boolean validQueenMove(BoardTile from,BoardTile target) {
    return validRookMove(from,target)||validBishopMove(from,target);
  }

  /** Checks the validity of a move for a Rook.
      @param target The target tile.
  */
  private boolean validRookMove(BoardTile from,BoardTile target) {
    int signx,signy,delta;

    try {  // Check that we don't eat our own piece
      if(target.getPiece().getSide()==
	 from.getPiece().getSide())
	return false;
    } catch(NullPointerException e) { /* Empty tile */ }
    

    if(target.xpos==from.xpos )
      return validStraightMove(from,0,(target.ypos-from.ypos)/
			       Math.abs(target.ypos-from.ypos),
			       Math.abs(target.ypos-from.ypos));
    else if(target.ypos==from.ypos)
      return validStraightMove(from,(target.xpos-from.xpos)/
			       Math.abs(target.xpos-from.xpos),0,
			       Math.abs(target.xpos-from.xpos));
			       
    return false;

  }


  /** This is a helpper method for pieces that move straight lines and
      diagonals.
      @param signx Direction to move on x acsis.
      @param singy Direction to move on y acsis.
      @param delta Length of the move. (Measured in (signx,signy))
      @return true if there's no pieces on the way.
  */
  private boolean validStraightMove(BoardTile from,int signx,int signy,int delta) {
    System.err.println(signx+":"+signy+"*"+delta);

    try {
      for(int i=1;i<Math.abs(delta);i++) {  //No pieces in between
	if(grid.getPiece(from.xpos+signx*i,from.ypos+signy*i)!=null)
	  return false;
      }
    } catch(ArrayIndexOutOfBoundsException e) { 
      System.err.println("Something seriously wrong in validStraightMove:"+e);
    }

    return true;
  
  }

  /** Moves the piece to target without any
      checking of validity of the move. This also removes eaten piece
      in en passant move. The move record is updated here because
      this is the only place we have enough information at hand.
      @param from Starting tile of the move.
      @param target The target tile where the piece is to be moved.
  */
  private void movePiece(BoardTile from,BoardTile target) {
    StringBuffer strb=new StringBuffer(16);
    boolean enPassant=(from.getPiece().getType() == Piece.PAWN &&
		       target.xpos != from.xpos &&
		       target.getPiece() == null);

    if(enPassant) {
      grid.setPiece(target.xpos,from.ypos,null);
    }

    if(from.getPiece().getTypeChar()!=0)
      strb.append(from.getPiece().getTypeChar());
    if(target.getPiece()!=null || enPassant) {
      if(from.getPiece().getType()==Piece.PAWN)
	strb.append(from.getColumnCode());
      strb.append('x');
    }
    strb.append(target);

    // we have to make the move before we can determine check.
    prevTarget=target;
    grid.setPiece(target.xpos,target.ypos,from.getPiece());
    grid.setPiece(from.xpos,from.ypos,null);

    //do we have a check?
    int otherSide=(target.getPiece().getSide()==Piece.WHITE?
		   Piece.BLACK:Piece.WHITE);
    if(isCheckPos(otherSide,grid.kingTiles(otherSide))!=null)
      strb.append('+');

    if(enPassant)
      strb.append(" e.p.");

    if(target.getPiece().getType()==Piece.KING) {
      if((target.xpos-from.xpos)==2)
	strb=new StringBuffer(16).append("o-o");
      else if((target.xpos-from.xpos)==-2)
	strb=new StringBuffer(16).append("o-o-o");
    }
    ctrl.addMove(whosTurn,strb);

  }

  /** Pause the timers. 
   */
  void pause() { ctrl.pause(); }
  /** Unpause the timers. This will start the white's timer if the window
      is iconified and deiconified before first move. (good or bad??)
   */
  void unpause() { ctrl.resumeTimer(whosTurn); }


  /* Chess Rules part */

  /** Checks if a piece in specific tile can move. Note: tile must
      contain a piece.
      @param tile Tile on the board, containing a piece.
  */
  private boolean canMove(BoardTile tile) {
    boolean result;
    BoardTile threat=null;

    try {
      threat=blocksThreatFrom(tile.getPiece().getSide(),tile);
    } catch(NullPointerException e) {
      System.err.println("Serious trouble in canMove():"+e);
    }

    if(threat!=null) {
      System.err.println("Terible threat in direction of "+threat);
    }

    switch(tile.getPiece().getType()) {
    case Piece.PAWN:
      result=canPawnMove(tile);

      /* If piece is blocking a threat, it eather can move to threat
	 or it can't move at all.
      */
      if(result && threat!=null)
	result=validPawnMove(tile,threat);
      break;
    case Piece.ROOK:
      result=canRookMove(tile);

      if(result && threat!=null)
	result=validRookMove(tile,threat);
      break;
    case Piece.BISHOP:
      result=canBishopMove(tile);

      if(result && threat!=null)
	result=validBishopMove(tile,threat);
      break;
    case Piece.QUEEN:
      result=canQueenMove(tile);

      if(result && threat!=null)
	result=validQueenMove(tile,threat);
      break;
    case Piece.KNIGHT:
      result=canKnightMove(tile);

      if(result && threat!=null)
	result=false; //It's just imposible for knight
      break;
    case Piece.KING:
      result=canKingMove(tile);
      break;
    default:
      System.out.println("jokumuu");
      result=false;
    }
    return result;
  }

  /** This checks if bishop,knight,rook or queen can move.
      @param tile Tile where the piece is.
      @param side Side of the moving piece.
      @param cords The relative cordinates where the piece can move.
      @see Piece.bishopMoves
      @see Piece.knightMoves
      @see Piece.rookMoves
  */
  private boolean canSimpleMove(BoardTile tile, int side, int cords[][]) {
   Piece tmp;
   
    for(int i=0;i<cords.length;i++) {
      try {
	tmp=grid.getPiece((tile.xpos+cords[i][0]),
			  (tile.ypos+cords[i][1]));
	if(tmp==null || tmp.getSide()!=side)
	  return true;
      } catch(ArrayIndexOutOfBoundsException e) { }
    }

    return false;
  }

  /** Checks if king on the tile can move.
      @param tile The tile where the knight is.
      @return true if can move, false if not.
  */
  private boolean canKingMove(BoardTile tile) {
    int kingMoves[][]={{-1,1},{0,1},{1,1},
		       {-1,0}, /**/ {1,0},
		       {-1,-1},{0,-1},{1,-1}};
    //if king can fortify, it can move to some of these.
    BoardTile moveTo;

    System.err.println("kingtile? "+tile);
    for(int i=0;i<kingMoves.length;i++) {
      try {
	moveTo=grid.getTile((tile.xpos+kingMoves[i][0]),
			    (tile.ypos+kingMoves[i][1]));

	if(validKingMove(tile,moveTo))
	  return true;
      } catch(ArrayIndexOutOfBoundsException e) {}
    }
    return false;
  }

  /** Checks if knight on the tile can move.
      @param tile The tile where the knight is.
      @return true if can move, false if not.
  */
  boolean canKnightMove(BoardTile tile) {
    return canSimpleMove(tile,tile.getPiece().getSide(),Piece.knightMoves);
  }

  /** Checks if rook on the tile can move.
      @param tile The tile where the rook is.
      @return true if can move, false if not.
  */
  boolean canRookMove(BoardTile tile) {
    return canSimpleMove(tile,tile.getPiece().getSide(),Piece.rookMoves);
  }
  /** Checks if queen on the tile can move.
      @param tile The tile where the queen is.
      @return true if can move, false if not.
  */
  boolean canQueenMove(BoardTile tile) {
    return canBishopMove(tile)||canRookMove(tile);
  }
  

  /** Checks if pawn on the tile can move.
      @param tile The tile where the pawn is.
      @return true if can move, false if not.
  */
  boolean canPawnMove(BoardTile tile) {
    int newRow=(tile.getPiece().getSide()==Piece.WHITE?1:-1)+tile.ypos;


    /* If the tiles that are tested are non-existent or null, this
       throws exception. */
    try {
    if(grid.getPiece(tile.xpos,newRow)==null)
      return true;
    } catch(ArrayIndexOutOfBoundsException e) {}
    try {
      if(grid.getPiece(tile.xpos+1,newRow).getSide() != 
	 tile.getPiece().getSide())
	return true;
    } catch(Exception e) {}
    try {
      if(grid.getPiece(tile.xpos-1,newRow).getSide() != 
	 tile.getPiece().getSide())
	return true;
    } catch(Exception e) {}

    // Can we en passant??
    if(prevTarget.getPiece().getType()==Piece.PAWN) {
      try {
	if(prevTarget.getPiece()==grid.getPiece(tile.xpos-1,tile.ypos) &&
	   grid.getPiece(tile.xpos-1,newRow)==null)
	  return true;
      } catch(ArrayIndexOutOfBoundsException e) {}
      try {
	if(prevTarget.getPiece()==grid.getPiece(tile.xpos+1,tile.ypos) &&
	   grid.getPiece(tile.xpos+1,newRow)==null)
	  return true;
      } catch(ArrayIndexOutOfBoundsException e) {}
    }
    return false;
  }

  /** Checks if bishop on the tile can move.
      @param tile The tile where the bishop is.
      @return true if can move, false if not.
  */
  boolean canBishopMove(BoardTile tile) {
    return canSimpleMove(tile,tile.getPiece().getSide(),Piece.bishopMoves);
  }


}
